import numpy as np
import matplotlib.pyplot as plt
from skimage.measure import shannon_entropy

img = plt.imread('../img/lezard.bmp')
h, w  = img.shape

H_scipy = shannon_entropy(img)  #marche aussi bien en float qu'en uint8 et non vectorisé

#%% 1ere façon de calculer l'histogramme 1 en 1 
ni = [] #ni est une liste
for i in range(0, 256):
    ni.append(np.sum(img==i)) #on compte le nombre de pixels à l'intensité i et on l'ajoute à la liste
ni = np.array(ni) #On transforme le tableau en array

#%% 2eme façon en utilisant np.histogram
#doc : par défaut 10 intervalles de même taille, se cale sur min max

ni, tmp = np.histogram(img, bins=256, range=[0,256])


#%% CALCUL DE L'ENTROPIE
# attention log défini que sur R+ ]0; +inf[
# nan = not a number (ambiguité mathématiques)

H = 0
for i in range(0,256):
    if (ni[i]>0):
        H = H - ni[i]/(h*w)*np.log2(ni[i]/(h*w))
print(H)

#%% 2eme façon de faire 
ni, tmp = np.histogram(img, bins=256, range=[0,256])
ni = np.delete(ni, ni==0) #% On enlève les coordonées à 0
H = - np.sum(ni*np.log2(ni/(h*w)))/(h*w)
print(H)

#%% 3eme façon de faire 
ni, tmp = np.histogram(img, bins=256, range=[0,256])
H = - np.sum(ni[ni>0]/(h*w)*np.log2(ni[ni>0]/(h*w))) #On ne prend que les valeurs où ni>0
print(H)

#%%  Matrice des différences colonne à colonne

img = img.astype('float') #On fait attention à convertir en double car on va faire des opérations
D = np.zeros((h,w))
D[:,0] = img[:,0]
D[:,1:] = img[:,1:] - img[:,0:-1]

plt.figure()
plt.imshow(D)
plt.title('Différence colonne à colonne')
plt.show()

#%%
plt.figure()
plt.hist(img.ravel(), 256) #ravel = vectorisation de l'image
plt.show()
plt.title('Histogramme de l''image')

plt.figure()
plt.hist(D.ravel(), 512)
plt.show()
plt.title('Histogramme de D')

#%% entropie
ni, tmp = np.histogram(D, bins=513, range=[-256,256])
H_D = - np.sum(ni[ni>0]/(h*w)*np.log2(ni[ni>0]/(h*w))) #On ne prend que les valeurs où ni>0
print(H_D)


#%% Reconstruction

imgr = np.zeros((h,w))
imgr[:,0] = D[:,0] #%recopie de la première colonne inchangée
for j in range(1,w):
    imgr[:,j] = imgr[:,j-1] + D[:,j] #%On rajoute la différence retranchée précédemment

plt.figure()
plt.imshow(imgr, cmap='gray')
plt.title('Image reconstruite')
plt.show()

diff = np.sum(np.abs(imgr - img))  #Les images sont bien les mêmes
print(diff)

